/*******************************************************************************

  TINY_TERMOMETER

  C/ PIC 12f675 e LCD (LPLEX)

  Autor: Cludio Lrios

  Data:  15/03/2017

  Objetivo: Medir temperaturas usando um LM35 na faixa de -50 a 150C.
  O dgito ps-virgula muda de 0,5 em 0,5 graus (converso em 10 bits com
  voltagem de referncia de 5 volts).
  
  Aplicao: Medio de temperatura ambiente, freeser, geladeiras,etc.

  Uso didtico

  Este arquivo  parte integrante do blog Lrios.Tecnologia.WS

*******************************************************************************/
#include <12F675.h>
#device adc=10
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, PUT, BROWNOUT

//bytes
#byte  trisio = 0x85
#byte  gpio   = 0x05


//pinos do lcd (metodo Lariosplex 4 FIOS)
#bit DB76  = 0x5.5  //pino 2 db7=direto db6=resistor 10k +cap 10nF ao gnd
#bit DB54  = 0x5.4  //pino 3 db5=direto db4=resistor 10k +cap 10nF ao gnd
#bit  E_RS = 0x5.2  //pino 5


#define vcarga 500 // por tentativa e erro (entre 200 a 500)
#define vdescarga 500//idem
#define v_delay_clk 2  // de 2 a 50 conforme LCD

#define V_REF_USADA 50000 //vcc (INTERNA) 
#define VALOR_FILTRO 10   //valor para filtrar oscilaes na leitura
//variveis globais
unsigned int1 RS,sig;
unsigned int8 cent,dez,un,vun,a,cnt_f;
unsigned int16 lido,res,res1,res_old;
unsigned int32 soma;

/*******************************************************************************
                          SUBROTINAS
*******************************************************************************/


void descarregar(){ DB76=0;DB54=0;E_RS=0; delay_us(vdescarga);}
void t_carga(){delay_us(vcarga);}
void clk(){ E_RS=1; delay_cycles(v_delay_clk); E_RS=0;}//gera clock par Lcd


/*******************************************************************************
                          SUBROTINAS de LCD
*******************************************************************************/


/*******************************************************************************
                         ENVIA NIBBLE
*******************************************************************************/
void env_nible(unsigned int8 nib){
    descarregar();//descarrega capacitores
    nib&=0x0f;//limita
    if(nib&0x01)DB54=1;//testa bit 0
    if(nib&0x04)DB76=1;//testa bit 2
    if(RS)E_RS=1;else E_RS=0;//comando ou dado
    t_carga();
    if(nib&0x02)DB54=1; else DB54=0;//testa bit 1
    if(nib&0x08)DB76=1; else DB76=0;//testa bit 3
    clk();//clock

}

/*******************************************************************************
                          ENVIA DADOS PARA O LCD
*******************************************************************************/

void env_lcd_d(unsigned int8 dado){

 RS=1;//indica que enviar 'dados'
 env_nible(dado>>4);//high nibble primeiro
 env_nible(dado);
 delay_ms(4);
}

/*******************************************************************************
                         ENVIA COMANDOS PARA LCD
*******************************************************************************/
void env_lcd_c(unsigned int8 comando){

 RS=0;//indica que enviar 'comandos'
 env_nible(comando>>4);//high nibble primeiro
 env_nible(comando);
 delay_ms(4);
}

/*******************************************************************************
                         INICIALIZAO DO LCD
*******************************************************************************/

void Lcd_Init(){
  unsigned int8 a;

   descarregar();//descarrega capacitores
   RS=0; //enviar comando
   delay_ms(40);//delay inicial
   for(a=0;a<3;a++) { env_nible(3);delay_ms(5);}//envia 3x 0x03
   env_nible(2);//envia 0x02
   delay_ms(5);//delay 5 ms
   env_lcd_c(0x28);//ajusta para 4 bits o modo de envio
   env_lcd_c(0x06);// no shift
   env_lcd_c(0x0c);//Display ON, Cursor OFF, Blink OFF
   env_lcd_c(0x01);//limpa tela
}

/*******************************************************************************
        FAZ LEITURA DE UM CANAL DO CONVERSOR ANALGICO/DIGITAL
*******************************************************************************/
void get_canal(unsigned int8 q){
     set_adc_channel(q); //ajusta para o pino 5 (AN2) ou  7 (AN0)
     delay_us(10);
     lido=0;// l valor da tenso de entrada do pino 5
     for(a=0;a<50;a++){//soma as 50 leituras sucessivas
     lido+=READ_ADC();
     }
     lido/=50;//mdia das ultimas 50 leituras
     soma= V_REF_USADA * (int32) lido/1023; // obtem a tenso de entrada c/ vref 5v



}

/*******************************************************************************
               MOSTRA NO LCD A LEITURA PROCESSADA
*******************************************************************************/
void mostra_res(){
  unsigned int1 ze=0;
     if(sig)env_lcd_d('-'); else env_lcd_d(' ');
     if(!cent){cent=240;ze=1;}//se for zero mostrar espao
     env_lcd_d(cent+'0');
     if(!dez && ze)dez=240;//apaga digito a esquerda da dezena
     env_lcd_d(dez+'0');
     env_lcd_d(un+'0');
     env_lcd_d(',');
     env_lcd_d(vun+'0');
}

/*******************************************************************************
                 CONVERTE DE HEXA PARA DECIMAL
*******************************************************************************/
void converte_res(){
    cent=res/10000;
    res=res%10000;
    dez=res/1000;
     res=res%1000;
     un=res/100;
     res=res%100;
     vun=res/10;


}
/*******************************************************************************
                 DELAY DE 100 MS
*******************************************************************************/
void delay_dbc(){
  delay_ms(100);
}




/*******************************************************************************
                           ROTINA MAIN
*******************************************************************************/
void main() {

  // setup_adc_ports(sAN0|VSS_VREF);
   setup_adc_ports(sAN0|VSS_VDD);//5volts
   setup_adc(ADC_CLOCK_DIV_8 );// ajusta o clock do adc
   set_adc_channel(0); //ajusta para o pino 7 (AN0)
   setup_counters(RTCC_INTERNAL,RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   trisio=0b001011;//ajusta entradas e sadas
   gpio=0;//zera tudo
   descarregar();//descarga dos capacitores do Lariosplex
   Lcd_Init();//inicializa Lcd
   printf(env_lcd_d("Tiny Termometer "));//mostra o nome
   env_lcd_c(0xc0);//linha 2
   printf(env_lcd_d("Larios.Tecnol.Ws"));// o blog
   delay_ms(2000);
   env_lcd_c(1);//apaga tela
   res_old=0;
   cnt_f=0;
/*******************************************************************************
                           ROTINA DE REPETIO
*******************************************************************************/

   for(;;){
     get_canal(0);// obtem resultado em 'soma'
     res=soma;//int32 para int16
     get_canal(1);// obtem resultado em 'soma'
     res1=soma;//int32 para int16
     
     sig=0; //positivo
     if(res>=res1)res=res-res1;
     else {res=res1 - res; sig=1;}
     
     if(res_old!=res){
       if(++cnt_f>VALOR_FILTRO){res_old=res;cnt_f=0;}
      
     }
     else{ cnt_f=0;}
     res=res_old;
     converte_res();// ajusta para decimal
     //envia resultado para lcd
     printf(env_lcd_d("Tiny Termometer "));
     env_lcd_c(0xc0); // 3 posico da linha 2
     printf(env_lcd_d("    "));//mostra "C"
     mostra_res(); //mostra resultado no LCD
     env_lcd_d(0xdf);// mostra simbolo de graus
     printf(env_lcd_d("C    "));//mostra "C"
     delay_ms(30);

   }
}
